import configparser
import math
from tensorflow.keras.models import Model  # pip install tensorflow (==2.3.0)
from tensorflow.keras.applications.resnet import preprocess_input
# from tensorflow.keras.preprocessing.image import img_to_array
# from tensorflow.keras.preprocessing.image import load_img
from tensorflow import keras
import os
# import argparse
import numpy as np
import cv2  # pip install opencv-python (==3.4.11.41)
import imutils  # pip install imutils
import skvideo.io
from tqdm import tqdm

# from SoccerNet import getListTestGames, getListGames

import torch
import torchvision
import json

from .utils import getListGames
# from .DataLoader import getDuration
from .Downloader import SoccerNetDownloader
from .DataLoader import Frame
from SoccerNet.DataLoaderTorch import ClipDataset, FrameDataset


class Extractor():
    def __init__(self, rootFolder,
                 feature="ResNET",
                 video="LQ",
                 back_end="TF2",
                 overwrite=False,
                 transform="crop",
                 tmp_HQ_videos=None,
                 grabber="skvideo"):
        self.rootFolder = rootFolder
        self.feature = feature
        self.video = video
        self.back_end = back_end
        self.verbose = True
        self.transform = transform
        self.overwrite = overwrite
        self.grabber = grabber

        self.tmp_HQ_videos = tmp_HQ_videos
        if self.tmp_HQ_videos:
            self.mySoccerNetDownloader = SoccerNetDownloader(self.rootFolder)
            self.mySoccerNetDownloader.password = self.tmp_HQ_videos

        if "TF2" in self.back_end:

            # create pretrained encoder (here ResNet152, pre-trained on ImageNet)
            base_model = keras.applications.resnet.ResNet152(include_top=True,
                                                             weights='imagenet',
                                                             input_tensor=None,
                                                             input_shape=None,
                                                             pooling=None,
                                                             classes=1000)

            # define model with output after polling layer (dim=2048)
            self.model = Model(base_model.input,
                               outputs=[base_model.get_layer("avg_pool").output])
            self.model.trainable = False

        elif self.back_end == "PT" and "ResNET" in self.feature:
            class Identity(torch.nn.Module):
                def __init__(self):
                    super(Identity, self).__init__()

                def forward(self, x):
                    return x

            self.model = torchvision.models.resnet152(
                pretrained=True, progress=True).cuda()
            self.model.fc = Identity()

        elif self.back_end == "PT" and "R25D" in self.feature:
            class Identity(torch.nn.Module):
                def __init__(self):
                    super(Identity, self).__init__()

                def forward(self, x):
                    return x

            self.model = torchvision.models.video.r2plus1d_18(
                pretrained=True, progress=True).cuda()
            self.model.fc = Identity()

        

    def extractAllGames(self):
        for i_game, game in tqdm(getListGames("all")):
            self.extractGameIndex(i_game)

    def extractGameIndex(self, index):
        print(getListGames("all")[index])
        if self.video =="LQ":
            for vid in ["1.mkv","2.mkv"]:
                self.extract(video_path=os.path.join(self.rootFolder, getListGames("all")[index], vid))

        elif self.video == "HQ":
            
            # read config for raw HD video
            config = configparser.ConfigParser()
            if not os.path.exists(os.path.join(self.rootFolder, getListGames("all")[index], "video.ini")) and self.tmp_HQ_videos is not None:
                self.mySoccerNetDownloader.downloadVideoHD(
                    game=getListGames("all")[index], file="video.ini")
            config.read(os.path.join(self.rootFolder, getListGames("all")[index], "video.ini"))

            # lopp over videos
            for vid in config.sections():
                video_path = os.path.join(self.rootFolder, getListGames("all")[index], vid)

                #Download video if does not exist, but remove it afterwards
                remove_afterwards = False
                if not os.path.exists(video_path) and self.tmp_HQ_videos is not None:
                    remove_afterwards = True
                    self.mySoccerNetDownloader.downloadVideoHD(game=getListGames("all")[index], file=vid)

                # extract feature for video
                self.extract(video_path=video_path,
                            start=float(config[vid]["start_time_second"]), 
                            duration=float(config[vid]["duration_second"]))
                
                # remove video if not present before
                if remove_afterwards:
                    os.remove(video_path)

    def extract(self, video_path, start=None, duration=None):
        print("extract video", video_path, "from", start, duration+start)
        # feature_path = video_path.replace(
        #     ".mkv", f"_{self.feature}_{self.back_end}.npy")
        feature_path = video_path[:-4] + f"_{self.feature}_{self.back_end}.npy"

        if os.path.exists(feature_path) and not self.overwrite:
            return
        if "TF2" in self.back_end:
            
            # Knowing number of frames from FFMPEG metadata w/o without iterating over all frames
            # videodata = skvideo.io.FFmpegReader(video_path)
            # (numframe, _, _, _) = videodata.getShape()  # numFrame x H x W x channels
            # if self.verbose:
            #     print("shape video", videodata.getShape())
            # time_second = getDuration(video_path)
            # fps_video = numframe / time_second

            # time_second = getDuration(video_path)
            # if self.verbose:
            #     print("duration video", time_second)

            # good_number_of_frames = False
            # while not good_number_of_frames:
            #     fps_video = numframe / time_second
            #     # time_second = numframe / fps_video

            #     frames = []
            #     videodata = skvideo.io.vreader(video_path)
            #     fps_desired = 2
            #     drop_extra_frames = fps_video/fps_desired

            #     for i_frame, frame in tqdm(enumerate(videodata), total=numframe):
            #         # print(i_frame % drop_extra_frames)
            #         if start is not None:
            #             if i_frame < fps_video * start:
            #                 continue

            #         if duration is not None:
            #             if i_frame > fps_video * (start + duration):
            #                 # print("end of duration :)")
            #                 continue

            #         if (i_frame % drop_extra_frames < 1):

            #             if self.crop_resize == "resize256crop224":  # crop keep the central square of the frame
            #                 frame = imutils.resize(frame, height=256)  # keep aspect ratio
            #                 # number of pixel to remove per side
            #                 off_side_h = int((frame.shape[0] - 224)/2)
            #                 off_side_w = int((frame.shape[1] - 224)/2)
            #                 frame = frame[off_side_h:-off_side_h,
            #                             off_side_w:-off_side_w, :]  # remove them

            #             elif self.crop_resize == "crop":  # crop keep the central square of the frame
            #                 frame = imutils.resize(frame, height=224)  # keep aspect ratio
            #                 # number of pixel to remove per side
            #                 off_side = int((frame.shape[1] - 224)/2)
            #                 frame = frame[:, off_side:-off_side, :]  # remove them

            #             elif self.crop_resize == "resize":  # resize change the aspect ratio
            #                 # lose aspect ratio
            #                 frame = cv2.resize(frame, (224, 224),
            #                                 interpolation=cv2.INTER_CUBIC)

            #             else:
            #                 raise NotImplmentedError()
            #             # if self.array:
            #             #     frame = img_to_array(frame)
            #             frames.append(frame)

            #     print("expected number of frames", numframe,
            #         "real number of available frames", i_frame+1)

            #     if numframe == i_frame+1:
            #         print("===>>> proper read! Proceeding! :)")
            #         good_number_of_frames = True
            #     else:
            #         print("===>>> not read properly... Read frames again! :(")
            #         numframe = i_frame+1
            if self.grabber=="skvideo":
                videoLoader = Frame(video_path, transform=self.transform, start=start, duration=duration)
            elif self.grabber=="opencv":
                videoLoader = FrameCV(video_path, transform=self.transform, start=start, duration=duration)

            # create numpy aray (nb_frames x 224 x 224 x 3)
            # frames = np.array(videoLoader.frames)
            # if self.preprocess:
            frames = preprocess_input(videoLoader.frames)
            
            if duration is None:
                duration = videoLoader.time_second
                # time_second = duration
            if self.verbose:
                print("frames", frames.shape, "fps=", frames.shape[0]/duration)

            # predict the featrues from the frames (adjust batch size for smalled GPU)
            features = self.model.predict(frames, batch_size=64, verbose=1)
            if self.verbose:
                print("features", features.shape, "fps=", features.shape[0]/duration)

        elif self.back_end == "PT" and "ResNET" in self.feature:

            myClips = FrameDataset(video_path=video_path,
                                  FPS_desired=2)

            myClipsLoader = torch.utils.data.DataLoader(myClips, batch_size=1)

            features = []
            for clip in tqdm(myClipsLoader):
                clip = clip.cuda()
                # print(clip.shape)
                feat = self.model(clip)
                # print(feat.shape)
                features.append(feat.detach().cpu())
                # print(len(features))
                # del clip
                # del feat
            print(orch.cat(features).shape)
            features = torch.cat(features).numpy()
            print(features.shape)


        elif self.back_end == "PT" and "R25D" in self.feature:

            myClips = ClipDataset(video_path=video_path, FPS_desired=2, clip_len=16)

            myClipsLoader = torch.utils.data.DataLoader(myClips, batch_size=1)


            features = []
            for clip in tqdm(myClipsLoader):
                clip = clip.cuda()
                # print(clip.shape)
                feat = self.model(clip)
                features.append(feat.detach().cpu())
                # print(len(feats))
                # del clip
                # del feat
            features = torch.cat(features).numpy()
            # print(features.shape)

        # save the featrue in .npy format
        os.makedirs(os.path.dirname(feature_path), exist_ok=True)
        np.save(feature_path, features)






import torch
# import pandas as pd


# class SoccerNetDataLoaderTorch(torch.utils.data.Dataset):
    
#     def __init__(self, soccernet_dir, jsonGamesFile=None, transform=None, input="Video_LQ"):
#         """
#         Args:
#             csv_file (string): Path to the csv file with annotations.
#             root_dir (string): Directory with all the images.
#             transform (callable, optional): Optional transform to be applied
#                 on a sample.
#         """

#         if jsonGamesFile is None:
#             jsonGamesFile = Path(__file__).parent / "data/SoccerNetGames.json"

#         self.jsonGamesFile = jsonGamesFile

#         with open(jsonGamesFile, "r") as json_file:
#             dictionary = json.load(json_file)

#         self.games = []
#         for championship in dictionary:
#             for season in dictionary[championship]:
#                 for game in dictionary[championship][season]:

#                     self.games.append(os.path.join(championship, season, game))
                    

#         # self.landmarks_frame = pd.read_csv(csv_file)
#         self.soccernet_dir = soccernet_dir
#         self.transform = transform
#         self.input = input

#     def __len__(self):
#         return len(self.games)

#     def __getitem__(self, idx):
#         game = self.games[idx]

#         if self.input == "Video_LQ":
#             labels = scikit.io.vread(os.path.join(self.games[idx], "Labels.txt")
#             video1 = scikit.io.vread(os.path.join(self.games[idx], "1.mkv")
#             video2 = scikit.io.vread(os.path.join(self.games[idx], "2.mkv")

#             sample={'video1': video1, 'video2': video2,
#                     'labels': labels}

#             if self.transform:
#                 sample = self.transform(sample)

#         return sample


import torch
import torchvision
import math
import os
import argparse
import numpy as np
import cv2  # pip install opencv-python (==3.4.11.41)
import imutils  # pip install imutils
# import skvideo.io
from tqdm import tqdm
import skvideo.io


def getDuration(video_path):
    metadata = skvideo.io.ffprobe(video_path)

    # read "@duration" metadata if availabel
    try:
        time_second = float(metadata["video"]["@duration"])
    except:
        pass

    # prioritize "video" "tag" "DURATION" metadata if readily available
    for entry in metadata["video"]["tag"]:
        if list(entry.items())[0][1] == "DURATION":
            # print("entry", entry)
            duration = list(entry.items())[1][1].split(":")
            # print(duration)
            time_second = int(duration[0])*3600 + \
                int(duration[1])*60 + float(duration[2])

    # print("duration video", time_second, video_path)
    return time_second


def video2clips(video, stride, clip_len):
    # nFrames = video.shape[0]
    start_idx = 0  # math.ceil(-clip_len/2)
    end_idx = video.shape[0] # - math.ceil(clip_len/2)
    idx = torch.arange(start=start_idx, end=end_idx, step=stride).long()
    idxs = []
    for i in torch.arange(math.ceil(-clip_len/2), math.ceil(clip_len/2)):
        idxs.append(idx+i)
    idx = torch.stack(idxs, dim=1)
    # idx = unfold2(idx, size=clip_len, step=stride)
    # print(idx)
    idx = idx.clamp(0, video.shape[0]-1)
    # print(idx)
    return video[idx]
    # i_frames = [for i in range()]
    # for i in range(nFrames):
    #     if i

    #     print(frame[])

# def resize(vid, size, interpolation='bilinear'):
#     # NOTE: using bilinear interpolation because we don't work on minibatches
#     # at this level
#     scale = None
#     if isinstance(size, int):
#         scale = float(size) / min(vid.shape[-2:])
#         size = None
#     return torch.nn.functional.interpolate(
#         vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)


def crop(vid, i, j, h, w):
    return vid[..., i:(i + h), j:(j + w)]


def center_crop(vid, output_size):
    h, w = vid.shape[-2:]
    th, tw = output_size

    i = int(round((h - th) / 2.))
    j = int(round((w - tw) / 2.))
    return crop(vid, i, j, th, tw)


def hflip(vid):
    return vid.flip(dims=(-1,))


# NOTE: for those functions, which generally expect mini-batches, we keep them
# as non-minibatch so that they are applied as if they were 4d (thus image).
# this way, we only apply the transformation in the spatial domain
def resize(vid, size, interpolation='bilinear'):
    # NOTE: using bilinear interpolation because we don't work on minibatches
    # at this level
    scale = None
    if isinstance(size, int):
        scale = float(size) / min(vid.shape[-2:])
        size = None
    return torch.nn.functional.interpolate(
        vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)


def pad(vid, padding, fill=0, padding_mode="constant"):
    # NOTE: don't want to pad on temporal dimension, so let as non-batch
    # (4d) before padding. This works as expected
    return torch.nn.functional.pad(vid, padding, value=fill, mode=padding_mode)


def to_normalized_float_tensor(vid):
    return vid.permute(3, 0, 1, 2).to(torch.float32) / 255


def normalize(vid, mean, std):
    shape = (-1,) + (1,) * (vid.dim() - 1)
    # shape = (1,) * (vid.dim() - 1) + (-1,)
    # print(vid.shape)
    mean = torch.as_tensor(mean).reshape(shape)
    # print(mean.shape)
    std = torch.as_tensor(std).reshape(shape)
    # print(std.shape)
    return (vid - mean) / std


class FrameDataset(torch.utils.data.Dataset):
    def __init__(self, video_path, FPS_desired):
        self.video_path=video_path
        self.FPS_desired = FPS_desired
        # self.clip_len = clip_len        

        # vframes, aframes, info = torchvision.io.read_video(
        #     video_path, pts_unit='sec')
        videodata = skvideo.io.FFmpegReader(video_path)
        (numframe, _, _, _) = videodata.getShape()
        videodata = skvideo.io.vreader(video_path)
        frames = []
        self.duration = getDuration(video_path)
        self.FPS = numframe/self.duration
        stride = self.FPS/self.FPS_desired
        for i_frame, frame in tqdm(enumerate(videodata), total=numframe):
            # frame = imutils.resize(frame, height=112)
            if (i_frame % stride < 1):
                # print(frame.shape)
                frame = imutils.resize(frame, height=224)  # keep aspect ratio
                # number of pixel to remove per side
                off_side = int((frame.shape[1] - 224)/2)
                frame = frame[:, off_side:-off_side, :]
                # print(frame.shape)
                # clip = resize(clip, (112, 112))
                # print("max frame intensity", np.max(frame))
                frames.append(frame)
        frames = np.array(frames)
        vframes = torch.from_numpy(frames)
        print(vframes.shape)
        vframes = vframes.permute([0, 3, 1, 2])


        print(vframes.shape)
        # normalize video
        # print("max vframes intensity", vframes.max())
        self.clips = vframes/255.0
        # print("max self.clips intensity", self.clips.max())
        # self.clips = normalize(vid=self.clips,
        #                     mean=[0.485, 0.456, 0.406],
        #                     std=[0.229, 0.224, 0.225])
        # shape = (1,)*(vframes.dim()-1) + (-1,)
        # mean = torch.as_tensor([0.43216, 0.394666, 0.37645]).reshape(shape)
        # std = torch.as_tensor([0.22803, 0.22145, 0.216989]).reshape(shape)
        # vframes = (vframes/255.0-mean)/std

        # create clips
        # self.clips = video2clips(vframes, stride=stride, clip_len=1)
        
        # permute and resize at need
        # print(vframes.shape)
        # self.clips = vframes.permute([0, 3, 1, 2]) #[:,:,0,:,:]
        # print(self.clips.shape)

    def __len__(self):
        return len(self.clips)



    def __getitem__(self, idx):
        # transform = torch.nn.functional.interpolate(
            # vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)
        clip = self.clips[idx, :, :, :]
        # print("clip", clip.shape)
        # clip = clip[:, :, ::2, 87:-87:2]
        # print(clip.shape)
        # clip = resize(clip, (224,224))
        # print(clip.shape)
        return clip


class ClipDataset(torch.utils.data.Dataset):
    def __init__(self, video_path, FPS_desired, clip_len):
        self.video_path=video_path
        self.FPS_desired = FPS_desired
        self.clip_len = clip_len        

        # vframes, aframes, info = torchvision.io.read_video(
        #     video_path, pts_unit='sec')
        videodata = skvideo.io.FFmpegReader(video_path)
        (numframe, _, _, _) = videodata.getShape()
        videodata = skvideo.io.vreader(video_path)
        frames = []
        for i_frame, frame in tqdm(enumerate(videodata), total=numframe):
            frame = imutils.resize(frame, height=224)
            off_side = int((frame.shape[1] - 224)/2)
            frame = frame[:, off_side:off_side+224, :]
            frame = imutils.resize(frame, height=112)  # keep aspect ratio
            # number of pixel to remove per side
            # clip = resize(clip, (112, 112))
            # print("max frame intensity", np.max(frame))
            frames.append(frame)
        frames = np.array(frames)
        vframes = torch.from_numpy(frames)

        self.duration = getDuration(video_path)
        self.FPS = vframes.shape[0]/self.duration

        # normalize video
        vframes = vframes/255.0
        # vframes = normalize(vid=vframes,
        #                     mean=[0.43216, 0.394666, 0.37645],
        #                     std=[0.22803, 0.22145, 0.216989])
        # shape = (1,)*(vframes.dim()-1) + (-1,)
        # mean = torch.as_tensor([0.43216, 0.394666, 0.37645]).reshape(shape)
        # std = torch.as_tensor([0.22803, 0.22145, 0.216989]).reshape(shape)
        # vframes = (vframes/255.0-mean)/std

        # create clips
        stride = self.FPS/self.FPS_desired
        self.clips = video2clips(
            vframes, stride=stride, clip_len=self.clip_len)
        # print(self.clips.shape)
        
        # permute and resize at need
        self.clips = self.clips.permute([0, 4, 1, 2, 3])
        # print(self.clips.shape)

    def __len__(self):
        return len(self.clips)



    def __getitem__(self, idx):
        # transform = torch.nn.functional.interpolate(
            # vid, size=size, scale_factor=scale, mode=interpolation, align_corners=False)
        clip = self.clips[idx, ...]
        # clip = clip[:, :, ::2, 87:-87:2]
        # print(clip.shape)
        # clip = resize(clip, (112,112))
        # print(clip.shape)
        return clip


if __name__ == "__main__":
    dataset = ClipDataset(
        "/media/giancos/Football/SoccerNet/england_epl/2014-2015/2015-02-21 - 18-00 Chelsea 1 - 1 Burnley/1.mkv", FPS_desired=2,clip_len=16)
    dataset = FrameDataset(
        "/media/giancos/Football/SoccerNet/england_epl/2014-2015/2015-02-21 - 18-00 Chelsea 1 - 1 Burnley/1.mkv", FPS_desired=2)




from pathlib import Path
import json
import os
try:
    import skvideo.io
except:
    pass
# import moviepy.editor

def getListGames(split="v1", task="spotting"):

    #  if single split, convert into a list
    if not isinstance(split, list):
        split = [split]

    # if an element is "v1", convert to  train/valid/test
    if "all" in split:
        split = ["train", "valid", "test", "challenge"]
    if "v1" in split:
        split.pop(split.index("v1"))
        split.append("train")
        split.append("valid")
        split.append("test")

    # if task == "spotting":
        
    listgames = []
    # print(split)
    # loop over splits
    for spl in split: 
        if task == "spotting":
            if spl == "train":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesTrain.json"
            elif spl == "valid":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesValid.json"
            elif spl == "test":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesTest.json"
            elif spl == "challenge":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetGamesChallenge.json"

        elif task == "camera-changes":
            if spl == "train":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesTrain.json"
            elif spl == "valid":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesValid.json"
            elif spl == "test":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesTest.json"
            elif spl == "challenge":
                jsonGamesFile = Path(__file__).parent / "data/SoccerNetCameraChangesChallenge.json"


        with open(jsonGamesFile, "r") as json_file:
            dictionary = json.load(json_file)

        for championship in dictionary:
            for season in dictionary[championship]:
                for game in dictionary[championship][season]:

                    listgames.append(os.path.join(championship, season, game))
                        
    return listgames


def getDuration(video_path):
    # metadata = skvideo.io.ffprobe(video_path)
    # print(video_path)
    # print("meta video", metadata["video"]["tag"])
    # print("meta audio", metadata["audio"]["tag"])
    # time_second = 0
    # try:
    #     # prioritize "video" "tag" "DURATION" metadata if readily available
    #     for entry in metadata["audio"]["tag"]:
    #         print("audio entry", list(entry.items()))
    #         if list(entry.items())[0][1] == "DURATION":
                
    #             duration = list(entry.items())[1][1].split(":")
    #             # print(duration)
    #             time_second = int(duration[0])*3600 + \
    #                 int(duration[1])*60 + float(duration[2])
    # except:
    #     pass

    # # read "@duration" metadata if availabel
    # try:
    #     time_second = float(metadata["video"]["@duration"])
    # except:
    #     pass

    # try:
    #     # prioritize "video" "tag" "DURATION" metadata if readily available
    #     for entry in metadata["video"]["tag"]:
    #         # print("video entry", list(entry.items()))
    #         if list(entry.items())[0][1] == "DURATION":
                
    #             duration = list(entry.items())[1][1].split(":")
    #             # print(duration)
    #             time_second = int(duration[0])*3600 + \
    #                 int(duration[1])*60 + float(duration[2])
    # except:
    #     pass
    # # print("duration video", time_second, video_path)
    # if abs(time_second - moviepy.editor.VideoFileClip(video_path).duration)>0.034:
    #     print(video_path)
    #     print("time_second", time_second)        
    #     print("moviepy", moviepy.editor.VideoFileClip(video_path).duration)    
        # assert False 
    time_second = moviepy.editor.VideoFileClip(video_path).duration
    return time_second

if __name__ == "__main__":
    print(len(getListGames(["v1"],task="camera-changes")))




import urllib.request
import os
from tqdm import tqdm
import json

from .utils import getListGames

class MyProgressBar():
    def __init__(self, filename):
        self.pbar = None
        self.filename = filename

    def __call__(self, block_num, block_size, total_size):
        if not self.pbar:
            self.pbar = tqdm(total=total_size, unit='iB', unit_scale=True)
            self.pbar.set_description(f"Downloading {self.filename}...")
            self.pbar.refresh()  # to show immediately the update

        self.pbar.update(block_size)



class OwnCloudDownloader():
    def __init__(self, LocalDirectory, OwnCloudServer):
        self.LocalDirectory = LocalDirectory
        self.OwnCloudServer = OwnCloudServer

    def downloadFile(self, path_local, path_owncloud, user=None, password=None, verbose=True):
        # return 0: successfully downloaded
        # return 1: HTTPError
        # return 2: unsupported error
        # return 3: file already exist locally

        if user is not None or password is not None:  
            # update Password
             
            password_mgr = urllib.request.HTTPPasswordMgrWithDefaultRealm()
            password_mgr.add_password(
                None, self.OwnCloudServer, user, password)
            handler = urllib.request.HTTPBasicAuthHandler(
                password_mgr)
            opener = urllib.request.build_opener(handler)
            urllib.request.install_opener(opener)

        if os.path.exists(path_local): # check existence
            if verbose:
                print(f"{path_local} already exists")
            return 2

        try:
            try:
                urllib.request.urlretrieve(
                    path_owncloud, path_local, MyProgressBar(path_local))

            except urllib.error.HTTPError as identifier:
                print(identifier)
                return 1
        except:
            os.remove(path_local)
            raise
            return 2
        return 0


class SoccerNetDownloader(OwnCloudDownloader):
    def __init__(self, LocalDirectory,
                 OwnCloudServer="https://exrcsdrive.kaust.edu.sa/exrcsdrive/public.php/webdav/"):
        super(SoccerNetDownloader, self).__init__(
            LocalDirectory, OwnCloudServer)

    def downloadVideoHD(self, game, file):
        # if file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:

        FileLocal = os.path.join(self.LocalDirectory, game, file)
        FileURL = os.path.join(self.OwnCloudServer, game, file).replace(' ', '%20')
        res = self.downloadFile(path_local=FileLocal,
                                path_owncloud=FileURL,
                                user="B72R7dTu1tZtIst",  # user for video HQ
                                password=self.password)


    def downloadVideo(self, game, file):
        # if file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:

        FileLocal = os.path.join(self.LocalDirectory, game, file)
        FileURL = os.path.join(self.OwnCloudServer, game, file).replace(' ', '%20')
        res = self.downloadFile(path_local=FileLocal,
                                path_owncloud=FileURL,
                                user="6XYClm33IyBkTgl",  # user for video
                                password=self.password)
                                    

    def downloadGames(self, files=["1.mkv", "2.mkv", "Labels.json"], split=["v1"], task="spotting", verbose=True):

        for spl in split:

        
            for game in getListGames(spl, task):


                for file in files:

                    GameDirectory = os.path.join(self.LocalDirectory, game)
                    FileURL = os.path.join(
                        self.OwnCloudServer, game, file).replace(' ', '%20')
                    os.makedirs(GameDirectory, exist_ok=True)

                    if spl == "challenge": # specific buckets for the challenge set

                        # LQ Videos
                        if file in ["1.mkv", "2.mkv"]:
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="trXNXsW9W04onBh",  # user for video LQ
                                                    password=self.password,
                                                    verbose=verbose)

                        # HQ Videos
                        elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="gJ8gja7V8SLxYBh",  # user for video HQ
                                                    password=self.password,
                                                    verbose=verbose)

                        # Labels
                        elif "Labels" in file:
                        # file in ["Labels.json", "Labels_v2.json"]:
                        # elif any(feat in file for feat in ["ResNET", "C3D", "I3D", "R25D"]):
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="WUOSnPSYRC1RY13",  # user for Labels
                                                    password=self.password,
                                                    verbose=verbose)

                        # Features
                        elif any(feat in file for feat in ["ResNET", "C3D", "I3D", "R25D"]):
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="d4nu5rJ6IilF9B0",  # user for Features
                                                    password="SoccerNet",
                                                    verbose=verbose)

                    else: # bucket for "v1"
                        # LQ Videos
                        if file in ["1.mkv", "2.mkv"]:
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="6XYClm33IyBkTgl",  # user for video LQ
                                                    password=self.password,
                                                    verbose=verbose)

                        # HQ Videos
                        elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="B72R7dTu1tZtIst",  # user for video HQ
                                                    password=self.password,
                                                    verbose=verbose)

                        # Labels
                        elif "Labels" in file:
                        # elif file in ["Labels.json"]:
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="ZDeEfBzCzseRCLA",  # user for Labels
                                                    password="SoccerNet",
                                                    verbose=verbose)
                        
                        # features
                        elif any(feat in file for feat in ["ResNET", "C3D", "I3D", "R25D"]):
                            res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
                                                    path_owncloud=FileURL,
                                                    user="9eRjic29XTk0gS9",  # user for Features
                                                    password="SoccerNet",
                                                    verbose=verbose)

    #     for file in files:

    #         GameDirectory = os.path.join(self.LocalDirectory, game)
    #         FileURL = os.path.join(
    #             self.OwnCloudServer, game, file).replace(' ', '%20')
    #         os.makedirs(GameDirectory, exist_ok=True)

    #         # LQ Videos
    #         if file in ["1.mkv", "2.mkv"]:
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="trXNXsW9W04onBh",  # user for video LQ
    #                                     password=self.password)

    #         # HQ Videos
    #         elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="gJ8gja7V8SLxYBh",  # user for video HQ
    #                                     password=self.password)

    #         # Labels
    #         elif file in ["Labels.json"]:
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="WUOSnPSYRC1RY13",  # user for Labels
    #                                     password=self.password)

    #         # Labels
    #         elif any(feat in file for feat in ["ResNET", "C3D", "I3D"]):
    #             res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                     path_owncloud=FileURL,
    #                                     user="d4nu5rJ6IilF9B0",  # user for Features
    #                                     password="SoccerNet")



    # def downloadTestGames(self, files=["1.mkv", "2.mkv", "Labels.json"]):

    #     for game in getListTestGames():

    #         # game = os.path.join(championship, season, game)


    #         for file in files:

    #             GameDirectory = os.path.join(self.LocalDirectory, game)
    #             FileURL = os.path.join(
    #                 self.OwnCloudServer, game, file).replace(' ', '%20')
    #             os.makedirs(GameDirectory, exist_ok=True)

    #             # LQ Videos
    #             if file in ["1.mkv", "2.mkv"]:
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="trXNXsW9W04onBh",  # user for video LQ
    #                                         password=self.password)

    #             # HQ Videos
    #             elif file in ["1_HQ.mkv", "2_HQ.mkv", "video.ini"]:
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="gJ8gja7V8SLxYBh",  # user for video HQ
    #                                         password=self.password)

    #             # Labels
    #             elif file in ["Labels.json"]:
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="WUOSnPSYRC1RY13",  # user for Labels
    #                                         password=self.password)

    #             # Labels
    #             elif any(feat in file for feat in ["ResNET", "C3D", "I3D"]):
    #                 res = self.downloadFile(path_local=os.path.join(GameDirectory, file),
    #                                         path_owncloud=FileURL,
    #                                         user="d4nu5rJ6IilF9B0",  # user for Features
    #                                         password="SoccerNet")


                    
